<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class HierarquiaUsuariosFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    //Busca lista de Hierarquias cadastradas
    public function getHierarquia()
    {
        $sql = " SELECT FCM_HIERARQUIAFUNC.NRHIERARQUIAFUNC, FCM_HIERARQUIAFUNC.NMHIERARQUIAFUNC, FCM_HIERARQUIAFUNC.IDATIVO, 
                        FCM_HIERARQUIAFUNC.NRLOCALCLIENTE
                    FROM FCM_HIERARQUIAFUNC
                   LEFT JOIN FCM_LOCALCLI 
                    ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_HIERARQUIAFUNC.NRLOCALCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // Busca os usuários da hierarquia escolhida
    public function getHierarquiaDetalhes($param)
    {
        $sql = "SELECT A.NRFUNCIONARIOSHIERARQ, A.CDOPERADOR, A.CDOPERADORPAI, B.NMOPERADOR, H.NMHIERARQUIAFUNC
                    FROM FCM_FUNCIONARIOSHIERARQ A
                    JOIN OPERADOR B ON B.CDOPERADOR = A.CDOPERADOR
                    JOIN FCM_HIERARQUIAFUNC H ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                    WHERE A.NRHIERARQUIAFUNC = :NRHIERARQUIAFUNC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    // Busca hierarquia de usuários que podem ser cadastrados.
    public function getHierarquiaUsuarios($param)
    {
        $sql = " SELECT CDOPERADOR, NMFUNCIONARIO
                    FROM FCM_FUNCIONARIO
                  WHERE CDOPERADOR NOT IN( SELECT A.CDOPERADOR
                                                            FROM FCM_FUNCIONARIO A
                                                                JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.CDOPERADOR = A.CDOPERADOR
                                                WHERE B.NRHIERARQUIAFUNC = :NRHIERARQUIAFUNC)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    // Busca uma lista personalizada de usuários que podem ser Pais na hierarquia
    public function getHierarquiaPais($param)
    {
        $sql = "SELECT CDOPERADOR, NMOPERADOR
                    FROM OPERADOR
                  WHERE CDOPERADOR IN( SELECT A.CDOPERADOR
                                                            FROM OPERADOR A
                                                                JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.CDOPERADOR = A.CDOPERADOR
                                              WHERE B.NRHIERARQUIAFUNC = :NRHIERARQUIAFUNC)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    // Cadastra novas Hierarquias
    public function salvarHierarquia($params)
    {

        $sql = "INSERT INTO FCM_HIERARQUIAFUNC(NRHIERARQUIAFUNC, NMHIERARQUIAFUNC, NRLOCALCLIENTE, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, CDOPERINCLUSAO, NRORGINCLUSAO,  CDOPERULTATU, NRORGULTATU)
        VALUES (:NRHIERARQUIAFUNC, :NMHIERARQUIAFUNC, :NRLOCALCLIENTE, :NRORG, :IDATIVO, SYSDATE, SYSDATE,  :CDOPERINCLUSAO, :NRORGINCLUSAO, :CDOPERULTATU, :NRORGULTATU)";


        $this->connection->executeStatement($sql, $params);
    }
    // Edita Hierarquias já cadastradas
    public function editarHierarquia($params)
    {
        $sql = "UPDATE  FCM_HIERARQUIAFUNC
                        SET IDATIVO             = :IDATIVO, 
                        NMHIERARQUIAFUNC        = :NMHIERARQUIAFUNC, 
                        NRORGULTATU             = :NRORGULTATU, 
                        CDOPERULTATU            = :CDOPERULTATU,
                        DTULTATU                = SYSDATE,
                        NRLOCALCLIENTE          = :NRLOCALCLIENTE
                WHERE NRHIERARQUIAFUNC  = :NRHIERARQUIAFUNC";


        $this->connection->executeStatement($sql, $params);
    }
    // Deleta hierarquias cadastradas e os usuários daquela hierarquia.
    public function excluirHierarquia($parametro)
    {
        $sq = "DELETE FROM FCM_FUNCIONARIOSHIERARQ
                WHERE NRHIERARQUIAFUNC IN ($parametro)";

        $this->connection->executeStatement($sq);

        $sql = "DELETE FROM FCM_HIERARQUIAFUNC
                WHERE NRHIERARQUIAFUNC IN ($parametro)";

        $this->connection->executeStatement($sql);
    }

    // Insere um novo usuário à hierarquia
    public function inserirUsuarios($params)
    {
        $sql = "INSERT INTO FCM_FUNCIONARIOSHIERARQ (NRFUNCIONARIOSHIERARQ, NRHIERARQUIAFUNC, CDOPERADOR, CDOPERADORPAI,   NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, DTINCLUSAO)
                                             VALUES (:NRFUNCIONARIOSHIERARQ, :NRHIERARQUIAFUNC, :CDOPERADOR, :CDOPERADORPAI, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO,  SYSDATE)";

        $this->connection->executeStatement($sql, $params);
    }

    // Deleta um usuário e sua ramificação abaixo
    public function excluirUsuarios($param)
    {

        $sql = "DELETE FROM FCM_FUNCIONARIOSHIERARQ
                    WHERE CDOPERADOR IN (" . $param . ")";

        $this->connection->executeStatement($sql);
    }

    public function buscarCdoperador($param)
    {
        $sql = "SELECT CDOPERADOR FROM FCM_FUNCIONARIOSHIERARQ 
        WHERE CDOPERADORPAI IN ( " . $param . " ) AND CDOPERADOR NOT IN ( " . $param . " )";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }
    
    // FUNÇÕES DA HIERARQUIA POR LOCAL
    public function getHierarquiaLocal()
    {
        $sql = "SELECT A.NRFUNCIONARIOSHIERARQ, A.CDOPERADOR, B.NMOPERADOR, H.NMHIERARQUIAFUNC, C.NMLOCALCLIENTE,
        DECODE(H.NRLOCALCLIENTE, NULL, A.CDOPERADORPAI, H.NRLOCALCLIENTE) AS CDOPERADORPAI 
                            FROM FCM_FUNCIONARIOSHIERARQ A
                            JOIN OPERADOR B ON B.CDOPERADOR = A.CDOPERADOR
                            JOIN FCM_HIERARQUIAFUNC H 
                            ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                            JOIN FCM_LOCALCLI C
                            ON C.NRLOCALCLIENTE = H.NRLOCALCLIENTE";
            
         return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getLocalHierarquia()
    {
        $sql = "SELECT FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE, 0 AS DISABLED 
                    FROM FCM_LOCALCLI
                    JOIN FCM_TIPOLOCALCLI
                    ON FCM_TIPOLOCALCLI.NRTIPOLOCALCLI = FCM_LOCALCLI.NRTIPOLOCALCLI
                    WHERE FCM_LOCALCLI.NRLOCALCLIENTE NOT IN (SELECT NRLOCALCLIENTE FROM FCM_HIERARQUIAFUNC WHERE NRLOCALCLIENTE IS NOT NULL)
                    AND FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = '1'
                    
                    UNION ALL 
                    
            SELECT FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE, 1 AS DISABLED 
                    FROM FCM_LOCALCLI
                    JOIN FCM_TIPOLOCALCLI
                    ON FCM_TIPOLOCALCLI.NRTIPOLOCALCLI = FCM_LOCALCLI.NRTIPOLOCALCLI
                    WHERE FCM_LOCALCLI.NRLOCALCLIENTE IN (SELECT NRLOCALCLIENTE FROM FCM_HIERARQUIAFUNC WHERE NRLOCALCLIENTE IS NOT NULL)
                    AND FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = '1'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
