<?php

namespace FM\Service;

use FM\Factory\HierarquiaFuncFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class HierarquiaFuncService
{

    protected $HierarquiaFuncFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(HierarquiaFuncFactory $HierarquiaFuncFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->HierarquiaFuncFactory        = $HierarquiaFuncFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    // Busca todas as hierarquias
    public function getHierarchies()
    {
        $hierarchiesList = $this->HierarquiaFuncFactory->getHierarchies();
        return $hierarchiesList;
    }

    // Busca o nr da hierarquia ativa
    public function getActiveHierarchyId()
    {
        $hierarchiesList = $this->HierarquiaFuncFactory->getActiveHierarchyId();

        return $hierarchiesList;
    }

    // Insere uma nova hierarquia
    public function insertHierarchy($params)
    {
        $nrhierarquiafunc  = $this->newCode->getCodeByTableName('FCM_HIERARQUIAFUNC', $this->Environment->getNrOrg(), 1, 10, true);

        $hierarchyParams = array(
            'NRHIERARQUIAFUNC' => $nrhierarquiafunc,
            'NMHIERARQUIAFUNC' => $params['nmhierarquiafunc'],
            'NRORG' =>  $this->Environment->getNrOrg()
        );
        try {
            $this->HierarquiaFuncFactory->insertHierarchy($hierarchyParams);
            $response = "Inserido com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
        }

        return $response;
    }

    // Ativa a hierarquia
    public function activateHierarchy($params)
    {

        $this->connection->beginTransaction();
        try {
            $this->HierarquiaFuncFactory->deactivateHierarchy(); // Desativa a hierarquia anterior
            $this->HierarquiaFuncFactory->activateHierarchy($params['idHierarchy']); // E ativa a atual

            $this->connection->commit();
            $response = "Ativado com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
            $this->connection->rollback();
        }

        return $response;
    }
}
