<?php

namespace FM\Service;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
use TCPDF;

class SendEmailTransferTask extends TCPDF
{
    const BASE_SUBJECT_EMAIL = 'Chave de acesso';
    const BASE_SUBJECT_EMAIL_ES = 'Clave de acceso';
    const BASE_SUBJECT_EMAIL_EN = 'Access key';

     /**
     * @param $style
     * @return string
     */
    private function getFailTransferTaskTemplatePath($style, $language = null) {
        if (empty($language)) {
            $language = 'pt_br';
        }
        $baseDir = __DIR__.'/../Util/EmailTemplates/';
        $languageDir = $baseDir . $language . '/';
        $customFile = $languageDir . 'fail_transfer_task' . $style . '.html';
        $defaultFile = $languageDir . 'fail_transfer_task.html';
        return file_exists($customFile) ? $customFile : $defaultFile;
    }

    /**
     * @param $cdOperator
     * @param $email
     * @param $password
     * @param $style
     * @return mixed|string
     */
    private function getReplacedTemplate($tasks, $email, $style, $language) {
        $replacedContents = utf8_decode(file_get_contents($this->getFailTransferTaskTemplatePath($style, $language)));

        if (count($tasks) == 1) {
            $task = $tasks[0];
            $details = '<p style="font-family: \'Open Sans\', arial !important; color:#ffffff; line-height:8px; font-size: 14px; font-weight: 600; line-height: 18px; margin-right:20px; margin-left:20px; margin-top:20px; margin-bottom:20px;">';
            $details .= 'Tarefa: ' . $task['NMCHECKLISTAGEND'] . '<br> Local: ' . $task['NMLOCALCLIENTE'] . '<br> Data e hora: ' . $task['DTHORAAGEND'] . '<br> Prioridade: ' . $task['NMPRIORIDADETAREFA'] . '<br> Operador atual: ' . $task['NMOPERADOR'] . '<br> Tempo de atraso: ' . $task['TEMPOATRASADODIAS'] . '<br> Atraso permitido: ' . $task['ATRASOPERMITIDO'] . '</p>';
            $replacedContents = str_replace('%taskDetails%', $details, $replacedContents);
        } else {
            $details = '<p style="font-family: \'Open Sans\', arial !important; color:#ffffff; line-height:8px; font-size: 14px; font-weight: 600; line-height: 18px; margin-right:20px; margin-left:20px; margin-top:20px; margin-bottom:20px;" >Histórico de tarefas atrasadas anexadas ao e-mail.</p></td>';
            $replacedContents = str_replace('%taskDetails%', $details, $replacedContents);
        }

        return $replacedContents;
        
        return $replacedContents;
    }

    public function sendEmail($tasks, $email, $style, $language)
    {
        $body = utf8_encode($this->getReplacedTemplate($tasks, $email, $style, $language));
        $smtpMail = InstanceManager::getInstance()->getService('smtpUtil');
        $subject = self::BASE_SUBJECT_EMAIL;
        if (strpos($language, 'es') !== false) {
            $subject = self::BASE_SUBJECT_EMAIL_ES;
        } else if (strpos($language, 'en') !== false) {
            $subject = self::BASE_SUBJECT_EMAIL_EN;
        }
        $smtpMail->setSubject($subject);
        $smtpMail->setIsHTML(true);

        if (count($tasks) > 1) {
            $pdfPath = $this->createPDF($tasks);
            $smtpMail->addAttachment($pdfPath);
        }

        $smtpMail->sendMail($body, $email);
    }

    // Adiciona a imagem no rodapé de todas as páginas
    public function Footer() {
        $this->SetY(-20); // Ajusta a posição do rodapé
        $imgFile = '../../frontend/public/assets/img/teknisa-facilities-vertical-institucional-preto.png';
        $this->Image($imgFile, 160, $this->GetY() + (-10), 40, 20, '', '', 'T', false, 300, '', false, false, 0, false, false, false);
    }

    private function createPDF($tasks)
    {
        $pdf = new self(); // Usa a própria classe para herdar o método Footer
        $pdf->AddPage();

        $html = '<h3>Tarefas atrasadas sem operadores disponíveis para transferência:  </h3>';
        foreach ($tasks as $task) {
            $html .= '<strong>Tarefa:</strong> ' . $task['NMCHECKLISTAGEND'] . '<br>';
            $html .= '<strong>Local:</strong> ' . $task['NMLOCALCLIENTE'] . '<br>';
            $html .= '<strong>Data e hora:</strong> ' . $task['DTHORAAGEND'] . '<br>';
            $html .= '<strong>Prioridade:</strong> ' . $task['NMPRIORIDADETAREFA'] . '<br>';
            $html .= '<strong>Operador atual:</strong> ' . $task['NMOPERADOR'] . '<br>';
            $html .= '<strong>Tempo de atraso:</strong> ' . $task['TEMPOATRASADODIAS'] . '<br>';
            $html .= '<strong>Atraso permitido:</strong> ' .$task['ATRASOPERMITIDO']  . '</p><br>';
        }

        $pdf->writeHTML($html, true, false, true, false, '');

        // Adiciona a imagem no rodapé (será chamada em todas as páginas automaticamente)
        $pdf->SetFooterData();
         
        $filePath = sys_get_temp_dir() . '/tarefas_atrasadas.pdf';
        $pdf->Output($filePath, 'F');

        return $filePath;
    }
}
