/* eslint-disable */
import {
  Metadata, Http,
} from '@zeedhi/core';
import {
  AlertService, Button, DialogService, LoadingService, Select, Text, TextInput, Tooltip,
} from '@zeedhi/common';

export class ControllerSensores {
  private contadorUnico: any;
  private contadorDuplo: any;
  private contadorGeral: any;

  private isItFirst: boolean = false;

  private changeButtons(cond: boolean, duplo: boolean = false) {
    Metadata.getInstance<Select>('SelectSensorEntrada').disabled = cond;
    if (duplo) {
      Metadata.getInstance<Select>('SelectSensorSaida').disabled = cond;
    }
    Metadata.getInstance<Button>('btnIniciarContador').disabled = cond;
    Metadata.getInstance<Button>('btnInterromperContador').iconName = (cond) ? 'mdi-pause' : 'mdi-reload';
    Metadata.getInstance<Tooltip>('tooltipCancelReset').label = (cond) ? 'Pausar contagem' : 'Reiniciar contagem';
  }

  public iniciarContagem() {
    if (window.location.pathname === '/SentidoUnico') {
      this.contagemSentidoUnico();
    } else if (window.location.pathname === '/SentidoDuplo') {
      this.contagemSentidoDuplo();
    } else if (window.location.pathname === '/SentidoGeral') {
      this.contagemSentidoGeral();
    }
  }

  public contagemSentidoUnico() {
    const ID = Metadata.getInstance<Select>('SelectSensorEntrada').value;
    const ID2 = Metadata.getInstance<Select>('SelectSensorSaida').value;
    if ((ID || ID !== null) && (ID2 || ID2 !== null)) {
      LoadingService.show('Inciando...');
      Http.get(`/getInicioEDateSensor?IP=${ID}&IP2=${ID2}`).then((response: any) => {
        LoadingService.hide();
        if (response.data.length && (response.data[0][0].IDMONITORAMENTO === 'S' || response.data[1][0].IDMONITORAMENTO === 'S')) {
          const DATAHORA = response.data[0][0].DATARESET.split(' ');
          DialogService.show({
            name: 'basic-dialog',
            type: 'info',
            title: 'Contagem encontrada',
            text: `Foi encontrada uma contagem já iniciada em um dos sensores selecionado.\nDeseja <b>reiniciar</b> a contagem ou <b>continuar</b> com os dados existente em ambos os sensores?`,
            buttons: [
              {
                name: 'zerar',
                component: 'ZdButton',
                outline: true,
                label: 'Reiniciar',
                events: {
                  click: () => this.iniciarContadorUnic('S'),
                },
              },
              {
                name: 'comtinuar',
                component: 'ZdButton',
                label: 'Continuar',
                events: {
                  click: () => this.iniciarContadorUnic('N'),
                },
              },
            ],
          });
        } else {
          this.iniciarContadorUnic('N');
        }
      });
    } else {
      this.alertMensagem('Selecione um sensor.', 'error');
    }
  }

  public contagemSentidoDuplo() {
    const ID = Metadata.getInstance<Select>('SelectSensorEntrada').value;
    if (ID || ID !== null) {
      LoadingService.show('Inciando...');
      Http.get(`/getInicioEDateSensor?IP=${ID}`).then((response: any) => {
        LoadingService.hide();
        if (response.data[0].IDMONITORAMENTO === 'S') {
          const DATAHORA = response.data[0].DATARESET.split(' ');
          DialogService.show({
            name: 'basic-dialog',
            type: 'info',
            title: 'Contagem encontrada',
            text: `Foi encontrada uma contagem já iniciada em ${DATAHORA[1]} às ${DATAHORA[0]}.\nDeseja <b>reiniciar</b> a contagem ou <b>continuar</b> com os dados existente?`,
            buttons: [
              {
                name: 'zerar',
                component: 'ZdButton',
                outline: true,
                label: 'Reiniciar',
                events: {
                  click: () => this.iniciarContadorDuplo('S'),
                },
              },
              {
                name: 'comtinuar',
                component: 'ZdButton',
                label: 'Continuar',
                events: {
                  click: () => this.iniciarContadorDuplo('N'),
                },
              },
            ],
          });
        } else {
          this.iniciarContadorDuplo('N');
        }
      });
    } else {
      this.alertMensagem('Selecione um sensor.', 'error');
    }
  }

  public contagemSentidoGeral() {
    const ID = Metadata.getInstance<Select>('SelectSensorEntrada').value;
    if (ID || ID !== null) {
      LoadingService.show('Inciando...');
      Http.get(`/getInicioEDateSensor?IP=${ID}`).then((response: any) => {
        LoadingService.hide();
        if (response.data[0].IDMONITORAMENTO === 'S') {
          const DATAHORA = response.data[0].DATARESET.split(' ');
          DialogService.show({
            name: 'basic-dialog',
            type: 'info',
            title: 'Contagem encontrada',
            text: `Foi encontrada uma contagem já iniciada em ${DATAHORA[1]} às ${DATAHORA[0]}.\nDeseja <b>reiniciar</b> a contagem ou <b>continuar</b> com os dados existente?`,
            buttons: [
              {
                name: 'zerar',
                component: 'ZdButton',
                outline: true,
                label: 'Reiniciar',
                events: {
                  click: () => this.iniciarContadorGeral('S'),
                },
              },
              {
                name: 'comtinuar',
                component: 'ZdButton',
                label: 'Continuar',
                events: {
                  click: () => this.iniciarContadorGeral('N'),
                },
              },
            ],
          });
        } else {
          this.iniciarContadorGeral('N');
        }
      });
    } else {
      this.alertMensagem('Selecione um sensor.', 'error');
    }
  }

  public async encerrarContagem({ component }:any) {
    const ID = Metadata.getInstance<Select>('SelectSensorEntrada').value;

    if(component.iconName === 'mdi-reload') {
      LoadingService.show('Reiniciando...');
      await Http.get(`/resetCount?IP=${ID}`);
      if (window.location.pathname === '/SentidoUnico') {
        const ID2 = Metadata.getInstance<Select>('SelectSensorSaida').value;
        await Http.get(`/resetCount?IP=${ID2}`);
        Metadata.getInstance<Text>('sentidoUnicoValorEntrada').text = '0';
        Metadata.getInstance<Text>('sentidoUnicoValorSaida').text = '0';
        Metadata.getInstance<Text>('sentidoUnicoValorTotal').text = '0';
      } else if (window.location.pathname === '/SentidoDuplo') {
        if (Metadata.getInstance<Text>('sentidoDuploValorTotal')) Metadata.getInstance<Text>('sentidoDuploValorTotal').text = '0';
        if (Metadata.getInstance<Text>('sentidoDuploValorEntradaSaida')) Metadata.getInstance<Text>('sentidoDuploValorEntradaSaida').text = '0';
        Metadata.getInstance<Text>('sentidoDuploValorTotal').text = '0';
      }
      if (window.location.pathname === '/SentidoGeral') Metadata.getInstance<Text>('sentidoGeralValorTotal').text = '0';
    } else {
      LoadingService.show('Pausando...');
      await Http.get(`/stopCount?IP=${ID}`);
      if (window.location.pathname === '/SentidoUnico') {
        const ID2 = Metadata.getInstance<Select>('SelectSensorSaida').value;
        await Http.get(`/stopCount?IP=${ID2}`);
        this.changeButtons(false, true);
        clearInterval(this.contadorUnico);
      } else {
        this.changeButtons(false);
        if (window.location.pathname === '/SentidoDuplo') {
          clearInterval(this.contadorDuplo);
        } else {
          clearInterval(this.contadorGeral);
        }
      }
    }
    LoadingService.hide();
  }
  
  public alertMensagem(mensagem: string, tipoMensagem: string = 'success') {
    AlertService.show({
      name: 'AlertInformativo',
      text: mensagem,
      timeout: 3000,
      top: true,
      left: false,
      color: tipoMensagem,
    })
  }
  
  public iniciarContadorGeral(statusReset: string) {
    this.isItFirst = true;
    DialogService.hide();
    this.changeButtons(true);
    const ID = Metadata.getInstance<Select>('SelectSensorEntrada').value;
    this.contadorGeral = setInterval(() => {
      Http.get(`/getCount?FIRST=${this.isItFirst}&RESET=${statusReset}&IDSENSOR=${ID}`).then((response: any) => {
        const valorEntrada = Metadata.getInstance<Text>('sentidoGeralValorTotal');
        valorEntrada.text = String(response.data);
      })
      this.isItFirst = false;
    }, 750);
  }
  
  public iniciarContadorDuplo(statusReset: string) {
    this.isItFirst = true;
    DialogService.hide();
    this.changeButtons(true);
    const ID = Metadata.getInstance<Select>('SelectSensorEntrada').value;
    const totalPessoa = Metadata.getInstance<TextInput>('limiteDePessoas').value;
    Metadata.getInstance<TextInput>('limiteDePessoas').disabled = true;
    this.contadorDuplo = setInterval(() => {
      Http.get(`/getCount?FIRST=${this.isItFirst}&RESET=${statusReset}&IDSENSOR=${ID}`).then((response: any) => {
        const valorEntrada = Metadata.getInstance<Text>('sentidoDuploValorEntradaSaida');
        valorEntrada.text = String(response.data);
        const valorOcupacao = Metadata.getInstance<Text>('sentidoDuploValorTotal');
        valorOcupacao.text = String(Math.floor(response.data / 2));
        if(Number(Math.floor(response.data / 2)) >= totalPessoa) {
          this.limiteAtigindo();
          clearInterval(this.contadorDuplo);
        }
      })
      this.isItFirst = false;
    }, 750);
  }

  public limiteAtigindo(){
    DialogService.show({
      name: 'basic-dialogXVP',
      type: 'warning',
      title: 'Limite Atingido',
      text: 'Limite de pessoas atingido. Deve ser liberado o local antes da entrada de novas pessoas.',
      cssStyle: {
        border: '2px solid red',
      },
      buttons: [
        {
          name: 'OK',
          component: 'ZdButton',
          label: 'OK',
          events: {
            click: '{{ControllerSensores.dialogRedirecionar}}',
          },
        }
      ],
    });
    (<any>document.getElementsByClassName('zd-dialog')[0]).style.border = '1px solid red';
  }

  public async dialogRedirecionar(){
    Metadata.getInstance<TextInput>('limiteDePessoas').disabled = false;
    this.changeButtons(false);
    DialogService.hide();
    (<any>document.getElementsByClassName('zd-dialog')[0]).style.border = '';
    this.encerrarContagem({ component: Metadata.getInstance<Button>('btnInterromperContador')});
  }
  
  public iniciarContadorUnic(statusReset: string) {
    this.isItFirst = true;
    DialogService.hide();
    this.changeButtons(true, true);
    const ID = Metadata.getInstance<Select>('SelectSensorEntrada').value;
    const ID2 = Metadata.getInstance<Select>('SelectSensorSaida').value;
    this.contadorUnico = setInterval(async() => {
      const valueEntry = await Http.get(`/getCount?FIRST=${this.isItFirst}&RESET=${statusReset}&IDSENSOR=${ID}`).then((response: any) => {
        return response.data;
      });
      const valueExit = await Http.get(`/getCount?FIRST=${this.isItFirst}&RESET=${statusReset}&IDSENSOR=${ID2}`).then((response: any) => {
        return response.data;
      })
      Metadata.getInstance<Text>('sentidoUnicoValorEntrada').text = String(valueEntry);
      Metadata.getInstance<Text>('sentidoUnicoValorSaida').text = String(valueExit);
      const occupation = 
      Metadata.getInstance<Text>('sentidoUnicoValorTotal').text = String(((valueEntry - valueExit) < 0) ? 0 : (valueEntry - valueExit));
      this.isItFirst = false;
    }, 750);
  }

  public encerrarInteval({ component }: any) {
    clearInterval(this.contadorGeral);
    clearInterval(this.contadorUnico);
    clearInterval(this.contadorDuplo);
  }

  public modifyBtnReset(){
    let filledField: boolean = true;
    switch(window.location.pathname){
      case('/SentidoDuplo' || '/SentidoGeral'):
        const selectDp = Metadata.getInstance<Select>('SelectSensorEntrada');
        filledField = (selectDp.value === null || selectDp.value === '');
        break;
      case('/SentidoUnico'):
        const selectUnEnt = Metadata.getInstance<Select>('SelectSensorEntrada');
        const selectUnSaid = Metadata.getInstance<Select>('SelectSensorSaida');
        filledField = (selectUnEnt.value === null || selectUnEnt.value === '') || (selectUnSaid.value === null || selectUnSaid.value === '');
        debugger;
        break;
    };
    Metadata.getInstance<Button>('btnInterromperContador').disabled = filledField;
  }
}
